package Test::Module;

use strict;
use warnings;

our $VERSION = '0.01';

sub __placeholder {}

1;
__END__

=head1 NAME

Test::Module - One line description

=for html
<a href="https://github.com/stevieb9/test-module/actions"><img src="https://github.com/stevieb9/test-module/workflows/CI/badge.svg"/></a>
<a href='https://coveralls.io/github/stevieb9/test-module?branch=master'><img src='https://coveralls.io/repos/stevieb9/test-module/badge.svg?branch=master&service=github' alt='Coverage Status' /></a>


=head1 SYNOPSIS

=head1 DESCRIPTION

=head1 METHODS

=head2 name

Description.

I<Parameters>:

    $bar

I<Mandatory, String>: The name of the thing with the guy and the place.

I<Returns>: C<0> upon success.

=head1 AUTHOR

Steve Bertrand, C<< <steveb at cpan.org> >>

=head1 LICENSE AND COPYRIGHT

Copyright 2021 Steve Bertrand.

This program is free software; you can redistribute it and/or modify it
under the terms of the the Artistic License (2.0). You may obtain a
copy of the full license at:

L<http://www.perlfoundation.org/artistic_license_2_0>
