package Test::Module::Second;

use warnings;
use strict;

our $VERSION = '0.01';

sub dummy {
    print "Test::Module::Second\n";
}

1;
__END__

=head1 NAME

One - The crappy ol' One module

=head1 SYNOPSIS

    use One;
    ...

=head1 DESCRIPTION

This module does something, but I forget what

=head1 FUNCTIONS

=head2 dummy

This sub does something
