package One;

use warnings;
use strict;

our $VERSION = '0.01';

sub dummy {
    print "One dummy\n";
}

1;
__END__

=head1 NAME

One - The crappy ol' One module

=head1 SYNOPSIS

    use One;
    ...

=head1 DESCRIPTION

This module does something, but I forget what

=head1 AUTHOR

Steve Bertrand, C<< <steveb at cpan.org> >>

=head1 LICENSE AND COPYRIGHT

Copyright 1999 Steve Bertrand.

This program is free software; you can redistribute it and/or modify it
under the terms of the the Artistic License (2.0). You may obtain a
copy of the full license at:

L<http://www.perlfoundation.org/artistic_license_2_0>

